/*
 * TextKeyMatcherView.java
 */

package textkeymatcher.ui;

import com.apple.eawt.AboutHandler;
import com.apple.eawt.AppEvent.AboutEvent;
import java.awt.Color;
import java.awt.Component;
import java.util.EventObject;
import org.apache.commons.lang3.StringUtils;
import org.jdesktop.application.Application.ExitListener;
import org.jdesktop.application.Task;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableCellRenderer;
import org.jdesktop.application.Application;
import textkeymatcher.TextKeyMatcherApp;
import textkeymatcher.entity.FilteredRowMap;
import textkeymatcher.entity.LineDataList;
import textkeymatcher.io.DocArchive;
import textkeymatcher.io.FileDocArchive;
import textkeymatcher.service.LineDataBuilder;
import textkeymatcher.service.LineDataBuilderFactory;
import textkeymatcher.service.LineDataCollector;
import textkeymatcher.service.LineDataLoader;
import textkeymatcher.ui.model.ImportDataDialogModel;

/**
 * The application's main frame.
 */
public class TextKeyMatcherView extends FrameView implements ExitListener {
    
    /**
     * ロガー
     */
    private static final Logger logger = Logger.getLogger(TextKeyMatcherView.class.getName());

    /**
     * テーブルの偶数行の背景色
     */
    private Color evenColor = new Color(240, 240, 255);
    
    /**
     * データがnullで、奇数行の場合の背景色
     */
    private Color nullValueColor = new Color(220, 220, 220);
    
    /**
     * データがnullで、偶数行の場合の背景色.<br>
     */
    private Color evenNullValueColor = new Color(210, 210, 240);
    
    
    public TextKeyMatcherView(SingleFrameApplication app) {
        super(app);

        initComponents();
        afterInitComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            @Override
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    /**
     * IDEで管理されないGUI要素の初期化
     */
    private void afterInitComponents() {
        // 終了アクション
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application
                .getInstance(textkeymatcher.TextKeyMatcherApp.class)
                .getContext().getActionMap(TextKeyMatcherView.class, this);
        
        javax.swing.Action actQuit = actionMap.get("quit"); // NOI18N
    
        // Macの場合
        if (System.getProperty("os.name").toLowerCase().indexOf("mac os x") >= 0) {
            // ウィンドウを閉じるキー(CMD-W)
            JRootPane root = getRootPane();
            InputMap im = root.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
            im.put(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.META_DOWN_MASK), actQuit);

            root.getActionMap().put(actQuit, actQuit);
            
            // ヘルプメニュー無効化
            helpMenu.setVisible(false);
            
            // アプリケーションハンドラ接続
            com.apple.eawt.Application app = com.apple.eawt.Application.getApplication();
            app.setAboutHandler(new AboutHandler() {
                @Override
                public void handleAbout(AboutEvent ae) {
                    showAboutBox();
                }
            });
        }
        
        // JTableのセルレンダー
        DefaultTableCellRenderer cellRender = new DefaultTableCellRenderer() {

            @Override
            public Component getTableCellRendererComponent(JTable table, Object value,
                    boolean isSelected, boolean hasFocus,
                    int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                
                if (isSelected) {
                    // フォーカスあり
                    setForeground(table.getSelectionForeground());
                    setBackground(table.getSelectionBackground());

                } else {
                    int colIndex = dataViewTable.convertColumnIndexToModel(column);
                    if (colIndex == 0 && dataViewTableModel.isTrailRow(row)) {
                        // キーカラムの連続キー行の場合、二つ目以降を灰色文字で表示する.
                        setForeground(Color.lightGray);

                    } else {
                        // それ意外は通常文字色
                        setForeground(table.getForeground());
                    }

                    // 有効なデータがある場合はキーごとに背景色を分ける
                    int serialNo = dataViewTableModel.getRowGroupSerialNo(row);
                    boolean even = (serialNo % 2 == 0);
                    if (value == null) {
                        // データがnullである場合の背景色
                        setBackground(even ? getEvenNullValueColor() : getNullValueColor());
                        
                    } else {
                        setBackground(even ? getEvenColor() : table.getBackground());
                    }
                }
                
                setHorizontalAlignment((value instanceof Number) ? RIGHT : LEFT);
                
                return this;
            }
        };
        dataViewTable.setDefaultRenderer(Object.class, cellRender);
    }

    /**
     * テーブルの偶数行の背景色を設定する.
     * @param evenColor 背景色
     */
    public void setEvenColor(Color evenColor) {
        if (evenColor == null) {
            throw new IllegalArgumentException();
        }
        Color oldValue = this.evenColor;
        this.evenColor = evenColor;
        firePropertyChange("evenColor", oldValue, evenColor);
    }
    
    /**
     * テーブルの偶数行の背景色を取得する.
     * @return 背景色
     */
    public Color getEvenColor() {
        return this.evenColor;
    }

    /**
     * データがnullで、偶数行の場合の背景色を設定する.
     * @param evenNullValueColor 背景色
     */
    public void setEvenNullValueColor(Color evenNullValueColor) {
        if (evenNullValueColor == null) {
            throw new IllegalArgumentException();
        }
        Color oldValue = this.evenNullValueColor;
        this.evenNullValueColor = evenNullValueColor;
        firePropertyChange("evenNullValueColor", oldValue, evenNullValueColor);
    }

    /**
     * データがnullで、偶数行の場合の背景色を設定する.
     * @return 背景色
     */
    public Color getEvenNullValueColor() {
        return this.evenNullValueColor;
    }
    
    /**
     * データがnullである場合の背景色を設定する.
     * @param nullValueColor 背景色
     */
    public void setNullValueColor(Color nullValueColor) {
        if (nullValueColor == null) {
            throw new IllegalArgumentException();
        }
        Color oldValue = this.nullValueColor;
        this.nullValueColor = nullValueColor;
        firePropertyChange("nullValueColor", oldValue, nullValueColor);
    }
    
    /**
     * データがnullである場合の背景色を取得する.
     * @return 背景色
     */
    public Color getNullValueColor() {
        return this.nullValueColor;
    }

    /**
     * アプリケーション情報ダイアログを表示する.
     */
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = TextKeyMatcherApp.getApplication().getMainFrame();
            aboutBox = new TextKeyMatcherAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        TextKeyMatcherApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        bindingGroup = new org.jdesktop.beansbinding.BindingGroup();

        mainPanel = new javax.swing.JPanel();
        dataViewTableSP = new javax.swing.JScrollPane();
        dataViewTable = new javax.swing.JTable();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        newMenuItem = new javax.swing.JMenuItem();
        openMenuItem = new javax.swing.JMenuItem();
        saveMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        dataMenu = new javax.swing.JMenu();
        loadDataMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        removeMenuItem = new javax.swing.JMenuItem();
        mergeMenuItem = new javax.swing.JMenuItem();
        keyMenu = new javax.swing.JMenu();
        textNormalRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        textIgnoreCaseRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        numberRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        viewMenu = new javax.swing.JMenu();
        allRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        matchedRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        unmatchedRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        existsLeftRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        missingLeftRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        uniqueRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        jRadioButtonMenuItem1 = new javax.swing.JRadioButtonMenuItem();
        ununiqueRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        jRadioButtonMenuItem2 = new javax.swing.JRadioButtonMenuItem();
        helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        viewButtonGroup = new javax.swing.ButtonGroup();
        keyButtonGroup = new javax.swing.ButtonGroup();
        dataViewTableModel = new textkeymatcher.ui.model.DataViewTableModel();
        keyMatcherBinder = new textkeymatcher.ui.model.KeyMatcherBinder();
        displayModeBinder = new textkeymatcher.ui.model.DisplayModeBinder();
        confirmRemoveColumnPanel = new javax.swing.JPanel();
        removeColumnLabel = new javax.swing.JLabel();
        removeColumnComboBox = new javax.swing.JComboBox();

        mainPanel.setName("mainPanel"); // NOI18N

        dataViewTableSP.setName("dataViewTableSP"); // NOI18N

        dataViewTable.setModel(dataViewTableModel);
        dataViewTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(textkeymatcher.TextKeyMatcherApp.class).getContext().getResourceMap(TextKeyMatcherView.class);
        dataViewTable.setGridColor(resourceMap.getColor("dataViewTable.gridColor")); // NOI18N
        dataViewTable.setName("dataViewTable"); // NOI18N
        dataViewTableSP.setViewportView(dataViewTable);

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(dataViewTableSP, javax.swing.GroupLayout.DEFAULT_SIZE, 435, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(dataViewTableSP, javax.swing.GroupLayout.DEFAULT_SIZE, 391, Short.MAX_VALUE)
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setMnemonic('F');
        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(textkeymatcher.TextKeyMatcherApp.class).getContext().getActionMap(TextKeyMatcherView.class, this);
        newMenuItem.setAction(actionMap.get("onNew")); // NOI18N
        newMenuItem.setText(resourceMap.getString("newMenuItem.text")); // NOI18N
        newMenuItem.setName("newMenuItem"); // NOI18N
        fileMenu.add(newMenuItem);

        openMenuItem.setAction(actionMap.get("onLoad")); // NOI18N
        openMenuItem.setText(resourceMap.getString("openMenuItem.text")); // NOI18N
        openMenuItem.setName("openMenuItem"); // NOI18N
        fileMenu.add(openMenuItem);

        saveMenuItem.setAction(actionMap.get("onSave")); // NOI18N
        saveMenuItem.setText(resourceMap.getString("saveMenuItem.text")); // NOI18N
        saveMenuItem.setName("saveMenuItem"); // NOI18N
        fileMenu.add(saveMenuItem);

        jSeparator1.setName("jSeparator1"); // NOI18N
        fileMenu.add(jSeparator1);

        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        dataMenu.setMnemonic('D');
        dataMenu.setText(resourceMap.getString("dataMenu.text")); // NOI18N
        dataMenu.setName("dataMenu"); // NOI18N

        loadDataMenuItem.setAction(actionMap.get("onImportData")); // NOI18N
        loadDataMenuItem.setText(resourceMap.getString("loadDataMenuItem.text")); // NOI18N
        loadDataMenuItem.setName("loadDataMenuItem"); // NOI18N
        dataMenu.add(loadDataMenuItem);

        jSeparator2.setName("jSeparator2"); // NOI18N
        dataMenu.add(jSeparator2);

        removeMenuItem.setAction(actionMap.get("onRemoveColumn")); // NOI18N
        removeMenuItem.setMnemonic('R');
        removeMenuItem.setText(resourceMap.getString("removeMenuItem.text")); // NOI18N
        removeMenuItem.setName("removeMenuItem"); // NOI18N
        dataMenu.add(removeMenuItem);

        mergeMenuItem.setMnemonic('M');
        mergeMenuItem.setText(resourceMap.getString("mergeMenuItem.text")); // NOI18N
        mergeMenuItem.setName("mergeMenuItem"); // NOI18N
        dataMenu.add(mergeMenuItem);

        menuBar.add(dataMenu);

        keyMenu.setMnemonic('K');
        keyMenu.setText(resourceMap.getString("keyMenu.text")); // NOI18N
        keyMenu.setName("keyMenu"); // NOI18N

        keyButtonGroup.add(textNormalRadioButtonMenuItem);
        textNormalRadioButtonMenuItem.setMnemonic('T');
        textNormalRadioButtonMenuItem.setText(resourceMap.getString("textNormalRadioButtonMenuItem.text")); // NOI18N
        textNormalRadioButtonMenuItem.setName("textNormalRadioButtonMenuItem"); // NOI18N

        org.jdesktop.beansbinding.Binding binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, keyMatcherBinder, org.jdesktop.beansbinding.ELProperty.create("${text}"), textNormalRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        keyMenu.add(textNormalRadioButtonMenuItem);

        keyButtonGroup.add(textIgnoreCaseRadioButtonMenuItem);
        textIgnoreCaseRadioButtonMenuItem.setMnemonic('I');
        textIgnoreCaseRadioButtonMenuItem.setText(resourceMap.getString("textIgnoreCaseRadioButtonMenuItem.text")); // NOI18N
        textIgnoreCaseRadioButtonMenuItem.setName("textIgnoreCaseRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, keyMatcherBinder, org.jdesktop.beansbinding.ELProperty.create("${textIgnoreCase}"), textIgnoreCaseRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        keyMenu.add(textIgnoreCaseRadioButtonMenuItem);

        keyButtonGroup.add(numberRadioButtonMenuItem);
        numberRadioButtonMenuItem.setMnemonic('N');
        numberRadioButtonMenuItem.setText(resourceMap.getString("numberRadioButtonMenuItem.text")); // NOI18N
        numberRadioButtonMenuItem.setName("numberRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, keyMatcherBinder, org.jdesktop.beansbinding.ELProperty.create("${numeric}"), numberRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        keyMenu.add(numberRadioButtonMenuItem);

        menuBar.add(keyMenu);

        viewMenu.setMnemonic('V');
        viewMenu.setText(resourceMap.getString("viewMenu.text")); // NOI18N
        viewMenu.setName("viewMenu"); // NOI18N

        viewButtonGroup.add(allRadioButtonMenuItem);
        allRadioButtonMenuItem.setMnemonic('A');
        allRadioButtonMenuItem.setText(resourceMap.getString("allRadioButtonMenuItem.text")); // NOI18N
        allRadioButtonMenuItem.setName("allRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${all}"), allRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(allRadioButtonMenuItem);

        viewButtonGroup.add(matchedRadioButtonMenuItem);
        matchedRadioButtonMenuItem.setMnemonic('M');
        matchedRadioButtonMenuItem.setText(resourceMap.getString("matchedRadioButtonMenuItem.text")); // NOI18N
        matchedRadioButtonMenuItem.setName("matchedRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${matched}"), matchedRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(matchedRadioButtonMenuItem);

        viewButtonGroup.add(unmatchedRadioButtonMenuItem);
        unmatchedRadioButtonMenuItem.setMnemonic('N');
        unmatchedRadioButtonMenuItem.setText(resourceMap.getString("unmatchedRadioButtonMenuItem.text")); // NOI18N
        unmatchedRadioButtonMenuItem.setName("unmatchedRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${unmatched}"), unmatchedRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(unmatchedRadioButtonMenuItem);

        viewButtonGroup.add(existsLeftRadioButtonMenuItem);
        existsLeftRadioButtonMenuItem.setMnemonic('1');
        existsLeftRadioButtonMenuItem.setText(resourceMap.getString("existsLeftRadioButtonMenuItem.text")); // NOI18N
        existsLeftRadioButtonMenuItem.setName("existsLeftRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${existsFirstColumn}"), existsLeftRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(existsLeftRadioButtonMenuItem);

        viewButtonGroup.add(missingLeftRadioButtonMenuItem);
        missingLeftRadioButtonMenuItem.setMnemonic('2');
        missingLeftRadioButtonMenuItem.setText(resourceMap.getString("missingLeftRadioButtonMenuItem.text")); // NOI18N
        missingLeftRadioButtonMenuItem.setName("missingLeftRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${missingFirstColumn}"), missingLeftRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(missingLeftRadioButtonMenuItem);

        viewButtonGroup.add(uniqueRadioButtonMenuItem);
        uniqueRadioButtonMenuItem.setMnemonic('U');
        uniqueRadioButtonMenuItem.setText(resourceMap.getString("uniqueRadioButtonMenuItem.text")); // NOI18N
        uniqueRadioButtonMenuItem.setName("uniqueRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${unique}"), uniqueRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(uniqueRadioButtonMenuItem);

        viewButtonGroup.add(jRadioButtonMenuItem1);
        jRadioButtonMenuItem1.setText(resourceMap.getString("jRadioButtonMenuItem1.text")); // NOI18N
        jRadioButtonMenuItem1.setName("jRadioButtonMenuItem1"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${uniqueAll}"), jRadioButtonMenuItem1, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(jRadioButtonMenuItem1);

        viewButtonGroup.add(ununiqueRadioButtonMenuItem);
        ununiqueRadioButtonMenuItem.setMnemonic('D');
        ununiqueRadioButtonMenuItem.setText(resourceMap.getString("ununiqueRadioButtonMenuItem.text")); // NOI18N
        ununiqueRadioButtonMenuItem.setName("ununiqueRadioButtonMenuItem"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${ununique}"), ununiqueRadioButtonMenuItem, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(ununiqueRadioButtonMenuItem);

        viewButtonGroup.add(jRadioButtonMenuItem2);
        jRadioButtonMenuItem2.setText(resourceMap.getString("jRadioButtonMenuItem2.text")); // NOI18N
        jRadioButtonMenuItem2.setName("jRadioButtonMenuItem2"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, displayModeBinder, org.jdesktop.beansbinding.ELProperty.create("${ununiqueAll}"), jRadioButtonMenuItem2, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        viewMenu.add(jRadioButtonMenuItem2);

        menuBar.add(viewMenu);

        helpMenu.setMnemonic('H');
        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setText(resourceMap.getString("aboutMenuItem.text")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 24));
        statusPanel.setName("statusPanel"); // NOI18N
        statusPanel.setPreferredSize(new java.awt.Dimension(404, 22));

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, statusPanelLayout.createSequentialGroup()
                .addComponent(statusMessageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(statusAnimationLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(statusMessageLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        keyMatcherBinder.bindDataViewTableModel(dataViewTableModel);

        displayModeBinder.bindDataViewTableModel(dataViewTableModel);

        confirmRemoveColumnPanel.setName("confirmRemoveColumnPanel"); // NOI18N
        confirmRemoveColumnPanel.setLayout(new java.awt.BorderLayout());

        removeColumnLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        removeColumnLabel.setText(resourceMap.getString("removeColumnLabel.text")); // NOI18N
        removeColumnLabel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        removeColumnLabel.setName("removeColumnLabel"); // NOI18N
        confirmRemoveColumnPanel.add(removeColumnLabel, java.awt.BorderLayout.NORTH);

        removeColumnComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        removeColumnComboBox.setName("removeColumnComboBox"); // NOI18N
        confirmRemoveColumnPanel.add(removeColumnComboBox, java.awt.BorderLayout.CENTER);

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);

        bindingGroup.bind();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * インポート設定ダイアログのモデル.<br>
     * 再表示時に前回使用したモデルが再利用される.<br>
     */
    private ImportDataDialogModel importDataDialogModel;

    /**
     * データのロードアクション.
     * @return 
     */
    @Action(block = Task.BlockingScope.WINDOW)
    public Task onImportData() {
        // データインポートダイアログの構築
        ImportDataDialog importDataDialog = new ImportDataDialog(getFrame(), true);
        importDataDialog.setLocationRelativeTo(importDataDialog.getParent());
        
        // 前回のデータを適用する.(あれば)
        if (importDataDialogModel != null) {
            importDataDialog.setModel(importDataDialogModel);
        }

        // 表示する.
        importDataDialog.setVisible(true);
        ImportDataDialogModel dlgModel = importDataDialog.getResultModel();
        if (dlgModel == null) {
            // キャンセルされた場合
            return null;
        }
        
        // 保存する.
        this.importDataDialogModel = dlgModel;
        
        return new OnImportDataTask(getApplication(), dlgModel);
    }

    /**
     * データロードの非同期ブロッキングアクション
     */
    private class OnImportDataTask extends org.jdesktop.application.Task<LineDataCollector, Void> {

        /**
         * インポートダイアログ
         */
        private ImportDataDialogModel dlgModel;
        
        /**
         * 補完されたタイトル.
         */
        private String title;

        
        OnImportDataTask(org.jdesktop.application.Application app, ImportDataDialogModel dlgModel) {
            // Runs on the EDT.  Copy GUI state that
            // doInBackground() depends on from parameters
            // to OnImportDataTask fields, here.
            super(app);
            this.dlgModel = dlgModel;

            // タイトルが空であれば適当につける
            String titleOrg = dlgModel.getTitle();
            if (StringUtils.isBlank(titleOrg)) {
                title = "(" + dataViewTableModel.getColumnCount() + ")";
            } else {
                title = titleOrg.trim();
            }
        }
        

        @Override protected LineDataCollector doInBackground() throws Exception {
            LineDataBuilderFactory factory = new LineDataBuilderFactory();
            LineDataBuilder builder = factory.createLineDataBuilder(dlgModel);
            
            LineDataCollector collector = new LineDataCollector(title);
            LineDataLoader loader = new LineDataLoader(builder, collector);
            
            if (dlgModel.isDirect()) {
                loader.load(dlgModel.getDirectText());

            } else {
                File sourceFile = new File(dlgModel.getSourceFile());
                loader.load(sourceFile, dlgModel.getCharset());
            }
            
            Thread.sleep(5000);
            
            return collector;
        }
        
        /**
         * 失敗した場合
         * @param ex 
         */
        @Override protected void failed(Throwable ex) {
            logger.log(Level.SEVERE, "データのインポートに失敗しました。", ex);
            statusMessageLabel.setText("failed.");
            ErrorDialog.showErrorDialog(ex);
        }

        @Override
        protected void cancelled() {
            logger.log(Level.INFO, "データのインポートをキャンセルしました。");
            statusMessageLabel.setText("cancelled.");
        }
        
        /**
         * 成功した場合
         * @param collector 
         */
        @Override protected void succeeded(LineDataCollector collector) {
            dataViewTableModel.addLineDataList(collector);
            checkEnableSave();
            checkEnableNew();
            checkEnableRemoveColumn();
            statusMessageLabel.setText("done.");
        }
    }

    /**
     * 終了することができるか必要であれば問い合わせる.<br>
     * @param event イベント
     * @return 終了できるならばtrue
     */
    @Override
    public boolean canExit(EventObject event) {
        logger.log(Level.INFO, "canExit?");
        return isConfirmDiscard();
    }

    /**
     * 終了されることを通知される.
     * @param event イベント
     */
    @Override
    public void willExit(EventObject event) {
        logger.log(Level.INFO, "ref={0}", new Object[] {FilteredRowMap.getRef()});
        logger.log(Level.INFO, "willExit");
    }

    /**
     * ドキュメントの破棄をダイアログで確認する.<br>
     * ドキュメントが空の場合は何も問い合わせずに直ちにtrueを返す.<br>
     * @return 破棄してもよい場合はtrue
     */
    private boolean isConfirmDiscard() {
        if ( !dataViewTableModel.isDirty()) {
            return true;
        }
        ResourceMap resourceMap = Application.getInstance()
                .getContext().getResourceMap(getClass());
        
        String title = resourceMap.getString("confirm.discard.title");
        String message = resourceMap.getString("confirm.discard.message");

        int ret = JOptionPane.showConfirmDialog(
                getFrame(),
                message,
                title,
                JOptionPane.YES_NO_OPTION,
                JOptionPane.QUESTION_MESSAGE
                );
        
        return ret == JOptionPane.YES_OPTION;
    }
    
    /**
     * 新規ドキュメントアクション
     */
    @Action(enabledProperty = "enableNew")
    public void onNew() {
        if ( !isConfirmDiscard()) {
            // キャンセル
            return;
        }
        dataViewTableModel.clear();
        dataViewTableModel.setDirty(false);

        checkEnableSave();
        checkEnableNew();
        checkEnableRemoveColumn();

        statusMessageLabel.setText("initialized.");
    }

    /**
     * 最後に使用したディレクトリ
     */
    private File lastUseDirectory;
    

    /**
     * データのロードアクション
     */
    @Action
    public void onLoad() {
        if (!isConfirmDiscard()) {
            // キャンセル
            return;
        }
        
        JFileChooser fileChooser = new JFileChooser(lastUseDirectory);
        int ret = fileChooser.showOpenDialog(getFrame());
        if (ret != JFileChooser.APPROVE_OPTION) {
            return;
        }
        
        dataViewTableModel.clear();

        File selFile = fileChooser.getSelectedFile();
        try{
            DocArchive archive = new FileDocArchive(selFile);
            dataViewTableModel.load(archive);
            dataViewTableModel.setDirty(false);
            statusMessageLabel.setText("loaded.");
            
        } catch (Exception ex) {
            logger.log(Level.SEVERE, "データのロードに失敗しました。" + selFile, ex);
            statusMessageLabel.setText("failed.");
            ErrorDialog.showErrorDialog(ex);
        }

        checkEnableSave();
        checkEnableNew();
        checkEnableRemoveColumn();
    }


    /**
     * 保存アクション
     */
    @Action(enabledProperty = "enableSave")
    public void onSave() {
        JFileChooser fileChooser = new JFileChooser(lastUseDirectory);
        int ret = fileChooser.showSaveDialog(getFrame());
        if (ret != JFileChooser.APPROVE_OPTION) {
            return;
        }
        
        File selFile = fileChooser.getSelectedFile();
        try {
            DocArchive archive = new FileDocArchive(selFile);
            dataViewTableModel.save(archive);
            dataViewTableModel.setDirty(false);
            statusMessageLabel.setText("saved.");

        } catch (Exception ex) {
            logger.log(Level.SEVERE, "saveで例外が発生しました。" + selFile, ex);
            statusMessageLabel.setText("failed.");
            ErrorDialog.showErrorDialog(ex);
        }
    }

    /**
     * カラムの削除アクション
     */
    @Action(enabledProperty = "enableRemoveColumn")
    public void onRemoveColumn() {
        int mx = dataViewTableModel.getNumOfLineDataLists();
        if (mx <= 1) {
            return;
        }
        
        // カラム一覧を作成する.
        ArrayList<Object> items = new ArrayList<Object>();
        for (int idx = 0; idx < mx; idx++) {
            final LineDataList lineDataList = dataViewTableModel.getLineDataList(idx);
            Object item = new Object() {
                @Override
                public String toString() {
                    return lineDataList.getTitle();
                }
            };
            items.add(item);
        }
        
        DefaultComboBoxModel comboboxModel = new DefaultComboBoxModel(
                items.toArray(new Object[items.size()]));

        removeColumnComboBox.setModel(comboboxModel);
        
        JOptionPane optionPane = new JOptionPane();
        
//        .showConfirmDialog(
//                getFrame(),
//                confirmRemoveColumnPanel,
//                "CONFIRM",
//                JOptionPane.YES_NO_OPTION,
//                JOptionPane.QUESTION_MESSAGE
//                );
    }

    private void checkEnableSave() {
        boolean hasDataSource = dataViewTableModel.getNumOfLineDataLists() > 0;
        setEnableSave(hasDataSource);
    }
    
    private void checkEnableNew() {
        boolean hasDataSource = dataViewTableModel.getNumOfLineDataLists() > 0;
        setEnableNew(hasDataSource);
    }
    
    private void checkEnableRemoveColumn() {
        // key/valueはペアであるため、最低1列は残す必要があるため、2以上の場合のみ削除可能.
        boolean hasDataSource = dataViewTableModel.getNumOfLineDataLists() > 1;
        setEnableRemoveColumn(hasDataSource);
    }

    private boolean enableSave = false;
    public boolean isEnableSave() {
        return enableSave;
    }

    public void setEnableSave(boolean b) {
        boolean old = isEnableSave();
        this.enableSave = b;
        firePropertyChange("enableSave", old, isEnableSave());
    }

    private boolean enableNew = false;
    public boolean isEnableNew() {
        return enableNew;
    }

    public void setEnableNew(boolean b) {
        boolean old = isEnableNew();
        this.enableNew = b;
        firePropertyChange("enableNew", old, isEnableNew());
    }

    private boolean enableRemoveColumn = false;
    public boolean isEnableRemoveColumn() {
        return enableRemoveColumn;
    }

    public void setEnableRemoveColumn(boolean b) {
        boolean old = isEnableRemoveColumn();
        this.enableRemoveColumn = b;
        firePropertyChange("enableRemoveColumn", old, isEnableRemoveColumn());
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JRadioButtonMenuItem allRadioButtonMenuItem;
    private javax.swing.JPanel confirmRemoveColumnPanel;
    private javax.swing.JMenu dataMenu;
    private javax.swing.JTable dataViewTable;
    private textkeymatcher.ui.model.DataViewTableModel dataViewTableModel;
    private javax.swing.JScrollPane dataViewTableSP;
    private textkeymatcher.ui.model.DisplayModeBinder displayModeBinder;
    private javax.swing.JRadioButtonMenuItem existsLeftRadioButtonMenuItem;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JRadioButtonMenuItem jRadioButtonMenuItem1;
    private javax.swing.JRadioButtonMenuItem jRadioButtonMenuItem2;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.ButtonGroup keyButtonGroup;
    private textkeymatcher.ui.model.KeyMatcherBinder keyMatcherBinder;
    private javax.swing.JMenu keyMenu;
    private javax.swing.JMenuItem loadDataMenuItem;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JRadioButtonMenuItem matchedRadioButtonMenuItem;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem mergeMenuItem;
    private javax.swing.JRadioButtonMenuItem missingLeftRadioButtonMenuItem;
    private javax.swing.JMenuItem newMenuItem;
    private javax.swing.JRadioButtonMenuItem numberRadioButtonMenuItem;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JComboBox removeColumnComboBox;
    private javax.swing.JLabel removeColumnLabel;
    private javax.swing.JMenuItem removeMenuItem;
    private javax.swing.JMenuItem saveMenuItem;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JRadioButtonMenuItem textIgnoreCaseRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem textNormalRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem uniqueRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem unmatchedRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem ununiqueRadioButtonMenuItem;
    private javax.swing.ButtonGroup viewButtonGroup;
    private javax.swing.JMenu viewMenu;
    private org.jdesktop.beansbinding.BindingGroup bindingGroup;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
}
